@@ -23,16 +23,18 @@
 #include <js/Exception.h>
 #include <js/GCHashTable.h>  // for GCHashMap
 #include <js/GCVector.h>     // for MutableWrappedPtrOperations
+#include <js/Id.h>
 #include <js/Object.h>       // for SetReservedSlot
 #include <js/PropertyAndElement.h>  // for JS_DefineFunction, JS_Enumerate
 #include <js/String.h>
-#include <js/TracingAPI.h>
 #include <js/TypeDecls.h>
 #include <js/Utility.h>  // for UniqueChars
 #include <js/Value.h>
 #include <js/ValueArray.h>
 #include <jsapi.h>  // for IdVector
 #include <mozilla/HashTable.h>
+#include <mozilla/Result.h>
+#include <mozilla/ScopeExit.h>
 #include <mozilla/Span.h>
 
 #include "gi/arg-inl.h"
@@ -57,6 +59,10 @@ template <GI::InfoTag TAG>
 [[nodiscard]]
 static bool struct_is_simple(const GI::UnownedInfo<TAG>&);
 
+template <GI::InfoTag TAG>
+[[nodiscard]]
+static bool simple_struct_has_pointers(const GI::UnownedInfo<TAG>&);
+
 template <class Base, class Prototype, class Instance>
 BoxedInstance<Base, Prototype, Instance>::BoxedInstance(Prototype* prototype,
                                                         JS::HandleObject obj)
@@ -105,7 +111,7 @@ template <class Base, class Prototype, class Instance>
 bool BoxedPrototype<Base, Prototype, Instance>::new_enumerate_impl(
     JSContext* cx, JS::HandleObject, JS::MutableHandleIdVector properties,
     bool only_enumerable [[maybe_unused]]) {
-    for (GI::AutoFunctionInfo meth_info : info().methods()) {
+    for (const GI::AutoFunctionInfo& meth_info : info().methods()) {
         if (meth_info.is_method()) {
             jsid id = gjs_intern_string_to_id(cx, meth_info.name());
             if (id.isVoid())
@@ -165,7 +171,7 @@ void BoxedInstance<Base, Prototype, Instance>::allocate_directly() {
 template <class Base, class Prototype, class Instance>
 std::unique_ptr<Boxed::FieldMap>
 BoxedPrototype<Base, Prototype, Instance>::create_field_map(
-    JSContext* cx, const BoxedInfo info) {
+    JSContext* cx, const BoxedInfo& info) {
     auto result = std::make_unique<Boxed::FieldMap>();
     typename BoxedInfo::FieldsIterator fields = info.fields();
     if (!result->reserve(fields.size())) {
@@ -347,7 +353,8 @@ bool BoxedInstance<Base, Prototype, Instance>::constructor_impl(
         if (g_type_is_a(gtype(), G_TYPE_BOXED)) {
             copy_boxed(source_priv->to_instance());
             return true;
-        } else if (get_prototype()->can_allocate_directly()) {
+        }
+        if (get_prototype()->can_allocate_directly()) {
             copy_memory(source_priv->to_instance());
             return true;
         }
@@ -356,19 +363,19 @@ bool BoxedInstance<Base, Prototype, Instance>::constructor_impl(
     Prototype* proto = get_prototype();
 
     // If the structure is registered as a boxed, we can create a new instance
-    // by looking for a zero-args constructor and calling it.
-    // Constructors don't really make sense for non-boxed types, since there is
-    // no memory management for the return value, and zero_args_constructor and
-    // default_constructor are always -1 for them.
+    // by looking for a zero-args constructor and calling it. Constructors don't
+    // really make sense for non-boxed types, since there is no memory
+    // management for the return value, and m_zero_args_constructor and
+    // m_default_constructor are always Nothing for them.
     //
     // For backward compatibility, we choose the zero args constructor if one
     // exists, otherwise we malloc the correct amount of space if possible;
     // finally, we fallback on the default constructor.
-    if (proto->has_zero_args_constructor()) {
-        GI::AutoFunctionInfo func_info{proto->zero_args_constructor_info()};
-
+    if (Maybe<GI::AutoFunctionInfo> zero_args_info{
+            proto->zero_args_constructor_info()};
+        zero_args_info) {
         GIArgument rval_arg;
-        Gjs::GErrorResult<> result = func_info.invoke({}, {}, &rval_arg);
+        Gjs::GErrorResult<> result = zero_args_info->invoke({}, {}, &rval_arg);
         if (result.isErr()) {
             gjs_throw(cx, "Failed to invoke boxed constructor: %s",
                       result.inspectErr()->message);
@@ -379,9 +386,14 @@ bool BoxedInstance<Base, Prototype, Instance>::constructor_impl(
 
         debug_lifecycle("Boxed pointer created from zero-args constructor");
 
-    } else if (proto->can_allocate_directly_without_pointers()) {
+    } else if (Maybe<GI::AutoFunctionInfo> default_ctor_info{
+                   proto->default_constructor_info()};
+               proto->can_allocate_directly_without_pointers() ||
+               (!default_ctor_info && proto->can_allocate_directly())) {
+        // has_default_constructor() takes priority over can_allocate_directly()
+        // for historical compatibility reasons
         allocate_directly();
-    } else if (proto->has_default_constructor()) {
+    } else if (default_ctor_info) {
         js::ESClass es_class = js::ESClass::Other;
         if (proto->can_allocate_directly() && args.length() == 1 &&
             args[0].isObject()) {
@@ -400,8 +412,10 @@ bool BoxedInstance<Base, Prototype, Instance>::constructor_impl(
             // constructor function (which we retrieve from the JS constructor,
             // that is, Namespace.BoxedType, or object.constructor, given that
             // object was created with the right prototype.
-            if (!invoke_static_method(cx, obj,
-                                      proto->default_constructor_name(), args))
+            JS::RootedId ctor_name{
+                cx, gjs_intern_string_to_id(cx, default_ctor_info->name())};
+            if (ctor_name.isVoid() ||
+                !invoke_static_method(cx, obj, ctor_name, args))
                 return false;
 
             // The return value of the JS constructor gets its own
@@ -412,8 +426,6 @@ bool BoxedInstance<Base, Prototype, Instance>::constructor_impl(
 
             return true;
         }
-    } else if (proto->can_allocate_directly()) {
-        allocate_directly();
     } else {
         gjs_throw(cx,
                   "Unable to construct struct type %s since it has no default "
@@ -464,6 +476,11 @@ BoxedInstance<Base, Prototype, Instance>::~BoxedInstance() {
     g_assert_not_reached();
 }
 
+template <class Base, class Prototype, class Instance>
+void BoxedInstance<Base, Prototype, Instance>::trace_impl(JSTracer* trc) {
+    m_nested_objects.trace(trc);
+}
+
 /**
  * BoxedBase::get_field_info:
  *
@@ -511,8 +528,8 @@ void adopt_nested_ptr(OtherInstance* priv, void* data) {
 template <class Base, class Prototype, class Instance>
 template <class FieldInstance>
 bool BoxedInstance<Base, Prototype, Instance>::get_nested_interface_object(
-    JSContext* cx, JSObject* parent_obj, const GI::FieldInfo field_info,
-    const GI::UnownedInfo<FieldInstance::TAG> struct_info,
+    JSContext* cx, JSObject* parent_obj, const GI::FieldInfo& field_info,
+    const GI::UnownedInfo<FieldInstance::TAG>& struct_info,
     JS::MutableHandleValue value) const {
     if (!struct_is_simple(struct_info)) {
         gjs_throw(cx, "Reading field %s.%s is not supported",
@@ -521,6 +538,14 @@ bool BoxedInstance<Base, Prototype, Instance>::get_nested_interface_object(
         return false;
     }
 
+    // If we have already set the field from a JS object which we have stashed
+    // because it owns pointers, return that JS object instead of creating one.
+    auto entry = m_nested_objects.lookup(field_info.name());
+    if (entry.found()) {
+        value.setObject(*entry->value().get());
+        return true;
+    }
+
     JS::RootedObject obj{
         cx, gjs_new_object_with_generic_prototype(cx, struct_info)};
     if (!obj)
@@ -570,7 +595,7 @@ bool BoxedBase<Base, Prototype, Instance>::field_getter(JSContext* cx,
 // See BoxedBase::field_getter().
 template <class Base, class Prototype, class Instance>
 bool BoxedInstance<Base, Prototype, Instance>::field_getter_impl(
-    JSContext* cx, JSObject* obj, const GI::FieldInfo field_info,
+    JSContext* cx, JSObject* obj, const GI::FieldInfo& field_info,
     JS::MutableHandleValue rval) const {
     GI::AutoTypeInfo type_info{field_info.type_info()};
 
@@ -579,7 +604,8 @@ bool BoxedInstance<Base, Prototype, Instance>::field_getter_impl(
         if (auto union_info = interface.as<GI::InfoTag::UNION>()) {
             return get_nested_interface_object<UnionInstance>(
                 cx, obj, field_info, union_info.value(), rval);
-        } else if (auto struct_info = interface.as<GI::InfoTag::STRUCT>()) {
+        }
+        if (auto struct_info = interface.as<GI::InfoTag::STRUCT>()) {
             return get_nested_interface_object<StructInstance>(
                 cx, obj, field_info, struct_info.value(), rval);
         }
@@ -630,12 +656,16 @@ bool BoxedInstance<Base, Prototype, Instance>::field_getter_impl(
  * method is called from BoxedInstance::field_setter_impl() when any such field
  * is being set. The contents of the BoxedInstance JS object in @value are
  * copied into the correct place in this BoxedInstance's memory.
+ *
+ * If the copied memory contains pointers, they are owned by the nested boxed
+ * type's JS object, so the nested JS object will be prevented from being
+ * garbage collected while the parent JS object is active.
  */
 template <class Base, class Prototype, class Instance>
 template <class FieldBase>
 bool BoxedInstance<Base, Prototype, Instance>::set_nested_interface_object(
-    JSContext* cx, const GI::FieldInfo field_info,
-    const GI::UnownedInfo<FieldBase::TAG> boxed_info, JS::HandleValue value) {
+    JSContext* cx, const GI::FieldInfo& field_info,
+    const GI::UnownedInfo<FieldBase::TAG>& boxed_info, JS::HandleValue value) {
     if (!struct_is_simple(boxed_info)) {
         gjs_throw(cx, "Writing field %s.%s is not supported",
                   format_name().c_str(), field_info.name());
@@ -646,19 +676,20 @@ bool BoxedInstance<Base, Prototype, Instance>::set_nested_interface_object(
     /* If we can't directly copy from the source object we need to construct a
      * new temporary object. */
     FieldBase* source_priv = nullptr;
+    JS::RootedObject source_object{cx};
     bool field_has_same_info = false;
 
     if constexpr (std::is_same_v<FieldBase, Base>) {
         field_has_same_info = info() == boxed_info;
         if (field_has_same_info) {
-            JS::RootedObject object{cx, &value.toObject()};
-            source_priv = FieldBase::for_js(cx, object);
+            source_object = &value.toObject();
+            source_priv = FieldBase::for_js(cx, source_object);
         }
     }
 
     if (!source_priv && !field_has_same_info) {
-        JS::RootedObject object{cx, &value.toObject()};
-        source_priv = FieldBase::for_js(cx, object);
+        source_object = &value.toObject();
+        source_priv = FieldBase::for_js(cx, source_object);
 
         if (source_priv && source_priv->info() != boxed_info) {
             std::string source_name{source_priv->format_name()};
@@ -676,16 +707,23 @@ bool BoxedInstance<Base, Prototype, Instance>::set_nested_interface_object(
 
         JS::RootedValueArray<1> args{cx};
         args[0].set(value);
-        JS::RootedObject tmp_object{
-            cx, gjs_construct_object_dynamic(cx, proto, args)};
-        if (!tmp_object ||
-            !FieldBase::for_js_typecheck(cx, tmp_object, &source_priv))
+        source_object = gjs_construct_object_dynamic(cx, proto, args);
+        if (!source_object ||
+            !FieldBase::for_js_typecheck(cx, source_object, &source_priv))
             return false;
     }
 
     if (!source_priv->check_is_instance(cx, "copy"))
         return false;
 
+    // Any pointers in the copied memory are still owned by the source struct.
+    // So we need to tie the lifetime of the source JS object to this one.
+    if (simple_struct_has_pointers(boxed_info) &&
+        !m_nested_objects.put(field_info.name(), source_object)) {
+        JS_ReportOutOfMemory(cx);
+        return false;
+    }
+
     memcpy(raw_ptr() + field_info.offset(), source_priv->to_instance()->ptr(),
            source_priv->info().size());
 
@@ -695,7 +733,7 @@ bool BoxedInstance<Base, Prototype, Instance>::set_nested_interface_object(
 // See BoxedBase::field_setter().
 template <class Base, class Prototype, class Instance>
 bool BoxedInstance<Base, Prototype, Instance>::field_setter_impl(
-    JSContext* cx, const GI::FieldInfo field_info, JS::HandleValue value) {
+    JSContext* cx, const GI::FieldInfo& field_info, JS::HandleValue value) {
     GI::AutoTypeInfo type_info{field_info.type_info()};
 
     if (!type_info.is_pointer() && type_info.tag() == GI_TYPE_TAG_INTERFACE) {
@@ -703,8 +741,8 @@ bool BoxedInstance<Base, Prototype, Instance>::field_setter_impl(
         if (auto union_info = interface_info.as<GI::InfoTag::UNION>()) {
             return set_nested_interface_object<UnionBase>(
                 cx, field_info, union_info.value(), value);
-        } else if (auto struct_info =
-                       interface_info.as<GI::InfoTag::STRUCT>()) {
+        }
+        if (auto struct_info = interface_info.as<GI::InfoTag::STRUCT>()) {
             return set_nested_interface_object<StructBase>(
                 cx, field_info, struct_info.value(), value);
         }
@@ -716,20 +754,21 @@ bool BoxedInstance<Base, Prototype, Instance>::field_setter_impl(
             GjsArgumentFlags::MAY_BE_NULL, field_info.name()))
         return false;
 
-    bool success = true;
+    auto cleanup = mozilla::MakeScopeExit([cx, &arg, &type_info]() {
+        JS::AutoSaveExceptionState saved_exc{cx};
+        if (!gjs_gi_argument_release(cx, GI_TRANSFER_NOTHING, type_info, &arg,
+                                     GjsArgumentFlags::ARG_IN))
+            gjs_log_exception(cx);
+        saved_exc.restore();
+    });
+
     if (field_info.write(m_ptr, &arg).isErr()) {
         gjs_throw(cx, "Writing field %s.%s is not supported",
                   format_name().c_str(), field_info.name());
-        success = false;
+        return false;
     }
 
-    JS::AutoSaveExceptionState saved_exc{cx};
-    if (!gjs_gi_argument_release(cx, GI_TRANSFER_NOTHING, type_info, &arg,
-                                 GjsArgumentFlags::ARG_IN))
-        gjs_log_exception(cx);
-    saved_exc.restore();
-
-    return success;
+    return true;
 }
 
 /**
@@ -763,7 +802,7 @@ bool BoxedBase<Base, Prototype, Instance>::field_setter(JSContext* cx,
 template <class Base, class Prototype, class Instance>
 std::string
 BoxedPrototype<Base, Prototype, Instance>::find_unique_js_field_name(
-    const BoxedInfo info, std::string const& c_field_name) {
+    const BoxedInfo& info, std::string const& c_field_name) {
     // Give priority to methods that have names equal to fields
     std::string property_name{c_field_name};
 
@@ -800,7 +839,7 @@ bool BoxedPrototype<Base, Prototype, Instance>::define_boxed_class_fields(
     //
     // At this point methods have already been defined on the prototype, so we
     // may get name conflicts which we need to check for.
-    for (GI::AutoFieldInfo field : info().fields()) {
+    for (const GI::AutoFieldInfo& field : info().fields()) {
         const std::string property_name =
             find_unique_js_field_name(info(), field.name());
         JS::RootedValue private_id{cx, JS::PrivateUint32Value(count++)};
@@ -827,8 +866,6 @@ bool BoxedPrototype<Base, Prototype, Instance>::define_boxed_class_fields(
 // Overrides GIWrapperPrototype::trace_impl().
 template <class Base, class Prototype, class Instance>
 void BoxedPrototype<Base, Prototype, Instance>::trace_impl(JSTracer* trc) {
-    JS::TraceEdge<jsid>(trc, &m_default_constructor_name,
-                        "Boxed::default_constructor_name");
     if (m_field_map)
         m_field_map->trace(trc);
 }
@@ -856,12 +893,8 @@ static bool type_can_be_allocated_directly(const GI::TypeInfo& type_info) {
     return false;
 }
 
-template <GI::InfoTag TAG>
 [[nodiscard]]
-static bool simple_struct_has_pointers(const GI::UnownedInfo<TAG>);
-
-[[nodiscard]]
-static bool direct_allocation_has_pointers(const GI::TypeInfo type_info) {
+static bool direct_allocation_has_pointers(const GI::TypeInfo& type_info) {
     if (type_info.is_pointer()) {
         if (type_info.tag() == GI_TYPE_TAG_ARRAY &&
             type_info.array_type() == GI_ARRAY_TYPE_C) {
@@ -877,7 +910,7 @@ static bool direct_allocation_has_pointers(const GI::TypeInfo type_info) {
     GI::AutoBaseInfo interface{type_info.interface()};
     if (auto struct_info = interface.as<GI::InfoTag::STRUCT>())
         return simple_struct_has_pointers(struct_info.value());
-    else if (auto union_info = interface.as<GI::InfoTag::UNION>())
+    if (auto union_info = interface.as<GI::InfoTag::UNION>())
         return simple_struct_has_pointers(union_info.value());
 
     return false;
@@ -904,87 +937,57 @@ bool struct_is_simple(const GI::UnownedInfo<TAG>& info) {
 
 template <GI::InfoTag TAG>
 [[nodiscard]]
-static bool simple_struct_has_pointers(const GI::UnownedInfo<TAG> info) {
+static bool simple_struct_has_pointers(const GI::UnownedInfo<TAG>& info) {
     g_assert(struct_is_simple(info) &&
              "Don't call simple_struct_has_pointers() on a non-simple struct");
 
     typename GI::UnownedInfo<TAG>::FieldsIterator fields = info.fields();
     return std::any_of(
-        fields.begin(), fields.end(), [](GI::AutoFieldInfo field) {
+        fields.begin(), fields.end(), [](const GI::AutoFieldInfo& field) {
             return direct_allocation_has_pointers(field.type_info());
         });
 }
 
 template <class Base, class Prototype, class Instance>
-BoxedPrototype<Base, Prototype, Instance>::BoxedPrototype(const BoxedInfo info,
+BoxedPrototype<Base, Prototype, Instance>::BoxedPrototype(const BoxedInfo& info,
                                                           GType gtype)
-    : BaseClass(info, gtype),
-      m_zero_args_constructor(-1),
-      m_default_constructor(-1),
-      m_default_constructor_name(JS::PropertyKey::Void()),
-      m_can_allocate_directly(struct_is_simple(info)) {
+    : BaseClass(info, gtype), m_can_allocate_directly(struct_is_simple(info)) {
     if (!m_can_allocate_directly) {
         m_can_allocate_directly_without_pointers = false;
     } else {
         m_can_allocate_directly_without_pointers =
             !simple_struct_has_pointers(info);
     }
-}
 
-// Overrides GIWrapperPrototype::init().
-template <class Base, class Prototype, class Instance>
-bool BoxedPrototype<Base, Prototype, Instance>::init(JSContext* cx) {
-    if (gtype() == G_TYPE_NONE)
-        return true;
+    if (gtype == G_TYPE_NONE)
+        return;
 
-    int i = 0;
-    int first_constructor = -1;
-    jsid first_constructor_name = JS::PropertyKey::Void();
-    jsid zero_args_constructor_name = JS::PropertyKey::Void();
+    ConstructorIndex i = 0;
+    Maybe<ConstructorIndex> first_constructor;
 
     /* If the structure is registered as a boxed, we can create a new instance
      * by looking for a zero-args constructor and calling it; constructors don't
      * really make sense for non-boxed types, since there is no memory
      * management for the return value.
      */
-    for (GI::AutoFunctionInfo func_info : info().methods()) {
+    for (const GI::AutoFunctionInfo& func_info : info.methods()) {
         if (func_info.is_constructor()) {
-            if (first_constructor < 0) {
-                first_constructor = i;
-                first_constructor_name =
-                    gjs_intern_string_to_id(cx, func_info.name());
-                if (first_constructor_name.isVoid())
-                    return false;
-            }
+            if (!first_constructor)
+                first_constructor = Some(i);
 
-            if (m_zero_args_constructor < 0 && func_info.n_args() == 0) {
-                m_zero_args_constructor = i;
-                zero_args_constructor_name =
-                    gjs_intern_string_to_id(cx, func_info.name());
-                if (zero_args_constructor_name.isVoid())
-                    return false;
-            }
+            if (!m_zero_args_constructor && func_info.n_args() == 0)
+                m_zero_args_constructor = Some(i);
 
-            if (m_default_constructor < 0 &&
-                strcmp(func_info.name(), "new") == 0) {
-                m_default_constructor = i;
-                const GjsAtoms& atoms = GjsContextPrivate::atoms(cx);
-                m_default_constructor_name = atoms.new_();
-            }
+            if (!m_default_constructor && strcmp(func_info.name(), "new") == 0)
+                m_default_constructor = Some(i);
         }
         i++;
     }
 
-    if (m_default_constructor < 0) {
+    if (!m_default_constructor && m_zero_args_constructor)
         m_default_constructor = m_zero_args_constructor;
-        m_default_constructor_name = zero_args_constructor_name;
-    }
-    if (m_default_constructor < 0) {
+    if (!m_default_constructor && first_constructor)
         m_default_constructor = first_constructor;
-        m_default_constructor_name = first_constructor_name;
-    }
-
-    return true;
 }
 
 /**
@@ -997,7 +1000,7 @@ bool BoxedPrototype<Base, Prototype, Instance>::init(JSContext* cx) {
  */
 template <class Base, class Prototype, class Instance>
 bool BoxedPrototype<Base, Prototype, Instance>::define_class_impl(
-    JSContext* cx, JS::HandleObject in_object, const BoxedInfo info,
+    JSContext* cx, JS::HandleObject in_object, const BoxedInfo& info,
     JS::MutableHandleObject prototype) {
     JS::RootedObject unused_constructor{cx};
     BoxedPrototype* priv = BoxedPrototype::create_class(
@@ -1011,7 +1014,7 @@ bool BoxedPrototype<Base, Prototype, Instance>::define_class_impl(
 template <class Base, class Prototype, class Instance>
 template <typename... Args>
 JSObject* BoxedInstance<Base, Prototype, Instance>::new_for_c_struct_impl(
-    JSContext* cx, const BoxedInfo info, void* gboxed, Args&&... args) {
+    JSContext* cx, const BoxedInfo& info, void* gboxed, Args... args) {
     if (gboxed == nullptr)
         return nullptr;
 
@@ -1058,11 +1061,13 @@ bool BoxedInstance<Base, Prototype, Instance>::init_from_c_struct(
     if (gtype() != G_TYPE_NONE && g_type_is_a(gtype(), G_TYPE_BOXED)) {
         copy_boxed(gboxed);
         return true;
-    } else if (gtype() == G_TYPE_VARIANT) {
+    }
+    if (gtype() == G_TYPE_VARIANT) {
         own_ptr(g_variant_ref_sink(static_cast<GVariant*>(gboxed)));
         debug_lifecycle("Boxed pointer created by sinking GVariant ref");
         return true;
-    } else if (get_prototype()->can_allocate_directly()) {
+    }
+    if (get_prototype()->can_allocate_directly()) {
         copy_memory(gboxed);
         return true;
     }
@@ -1075,14 +1080,13 @@ bool BoxedInstance<Base, Prototype, Instance>::init_from_c_struct(
 template class BoxedBase<StructBase, StructPrototype, StructInstance>;
 template class BoxedPrototype<StructBase, StructPrototype, StructInstance>;
 template class BoxedInstance<StructBase, StructPrototype, StructInstance>;
-template JSObject* StructInstance::new_for_c_struct_impl<>(JSContext*,
-                                                           const GI::StructInfo,
-                                                           void*);
+template JSObject* StructInstance::new_for_c_struct_impl<>(
+    JSContext*, const GI::StructInfo&, void*);
 template JSObject* StructInstance::new_for_c_struct_impl<Boxed::NoCopy>(
-    JSContext*, const GI::StructInfo, void*, Boxed::NoCopy&&);
+    JSContext*, const GI::StructInfo&, void*, Boxed::NoCopy);
 template class BoxedBase<UnionBase, UnionPrototype, UnionInstance>;
 template class BoxedPrototype<UnionBase, UnionPrototype, UnionInstance>;
 template class BoxedInstance<UnionBase, UnionPrototype, UnionInstance>;
 template JSObject* UnionInstance::new_for_c_struct_impl<>(JSContext*,
-                                                          const GI::UnionInfo,
+                                                          const GI::UnionInfo&,
                                                           void*);
